﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
// Example of using IviPwrMeter interface to control NRP2 base unit with 2 powersensors connected to inputs A (CH1) and B (CH2)
// To use the namespace RohdeSchwarz.RsPwrMeter, add the reference to the last RohdeSchwarz.RsPwrMeter.Fx40 assembly from either:
// "C:\Program Files\IVI Foundation\IVI\Microsoft.NET\Framework64\v4.0.30319\RohdeSchwarz.RsPwrMeter 1.4.0.0\Bin\RohdeSchwarz.RsPwrMeter.Fx40.dll"
// or
// "C:\Program Files (x86)\IVI Foundation\IVI\Microsoft.NET\Framework32\v4.0.30319\RohdeSchwarz.RsPwrMeter 1.4.0.0\Bin\RohdeSchwarz.RsPwrMeter.Fx40.dll"
using RohdeSchwarz.RsPwrMeter;
//to use the namespaces Ivi.Driver and Ivi.PwrMeter, install the:
//- IVI Shared Components 2.4.2 or newer
//- IVI.NET Shared Components 1.4.0
//download link: http://www.ivifoundation.org/shared_components/
using Ivi.Driver; //assembly must be of version 1.4.0.0
using Ivi.PwrMeter; //assembly must be of version 1.4.0.0

namespace NRP2_ReadChannel
{
    class Program
    {
        static void Main(string[] args)
        {
            //Before running, change the resource name to fit your device:
            //For LAN interface use TCPIP::<IP_Address>::INSTR e.g. TCPIP::10.85.0.84::INSTR
            //For GPIB interface use the GPIB::<Address>::INSTR e.g.GPIB::28::INSTR 
            var resourceName = "TCPIP::10.85.0.84::INSTR";
            IIviPwrMeter pwrmeter = null;
            // try-catch for opening the session
            try
            {
                Console.WriteLine($"Opening the session: '{resourceName}'");
                pwrmeter = new RsPwrMeter(resourceName, true, true) as IIviPwrMeter;
                Console.WriteLine("Success\n");
            }
            catch (Exception e)
            {
                Console.WriteLine($"Error during Opening the session:\n{e.Message}");
                Console.WriteLine("Press any key to finish");
                Console.ReadKey();
                return;
            }

            // try-catch after opening the session
            try
            {
                //setting the units for all the channels
                pwrmeter.Channels.Units = Ivi.PwrMeter.Units.dBm;
                Console.Write($"Zeroing the CH1 powersensor... ");
                pwrmeter.Channels["CH1"].Zero();
                Console.WriteLine("Success\n");

                Console.Write($"Zeroing the CH2 powersensor... ");
                pwrmeter.Channels["CH2"].Zero();
                Console.WriteLine("Success\n");

                Console.Write($"Zeroing all available powersensors in parallel... ");
                pwrmeter.Channels.Zero();
                Console.WriteLine("Success\n");

                //setting the range limits
                //if the measured value is above the Upper limit, the driver returns +Inf
                pwrmeter.Channels["CH1"].Range.Upper = 30.0;
                //if the measured value is below the Lower limit, the driver returns +-Inf
                pwrmeter.Channels["CH1"].Range.Lower = -70.0;

                #region Starting measurements on individual sensors
                Console.WriteLine("Initiating a measurement on the CH1 only, reading the result... ");
                var readValue1 = pwrmeter.Measurement.ReadChannel("CH1", new PrecisionTimeSpan(10));
                Console.WriteLine($"Channel 1 power: {readValue1}\n");

                //Initiating a measurement on the CH2 only, and reading the result
                Console.WriteLine("Initiating a measurement on the CH2 only, reading the result... ");
                var readValue2 = pwrmeter.Measurement.ReadChannel("CH2", new PrecisionTimeSpan(10));
                Console.WriteLine($"Channel 2 power: {readValue2}\n");
                #endregion

                #region Starting measurements on all sensors in parallel - faster than reading sensors individually
                Console.WriteLine("Initiating a measurement on all available channels in parallel");
                //the driver does not wait in the method Initiate() until the measurements are finished
                //the actual waiting is done in FetchChannel() methods
                pwrmeter.Measurement.Initiate();
                Console.WriteLine("Fetching the result for CH1 and CH2... ");
                readValue1 = pwrmeter.Measurement.FetchChannel("CH1");
                readValue2 = pwrmeter.Measurement.FetchChannel("CH2");
                Console.WriteLine($"Channel 1 power: {readValue1}");
                Console.WriteLine($"Channel 2 power: {readValue2}");
                #endregion
            }
            catch (Ivi.Driver.InstrumentStatusException e)
            {
                Console.WriteLine($"\nInstrument status error:\n{e.Message}");
            }
            finally
            {
                Console.WriteLine("\n\nPress any key to finish");
                Console.ReadKey();
            }
        }
    }
}
