﻿/////////////////////////////////////////////////////////////////////////////
// Name:        ReadSinglePower
//
// Purpose:     Sample program to demonstrate RF power measurement 
//              using an R&S NRPxxS series power sensor by utilizing
//              the R&S IVI.NET PowerMeter driver under C#
//
// Author:      J. Geltinger
// Created:     25-MAR-2015
// Modified by: M. Macko
// Modified:    25-JUN-2017
// Copyright:   (c) Rohde & Schwarz, Munich
/////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Globalization;
using RohdeSchwarz.RsPwrMeter;

//--------------------------------------------------------------------------
// Prerequisites
//
//   This sample program needs IVI Shared Components and IVI.NET Shared Components being installed.
//   If not yet done, visit
//     http://www.ivifoundation.org/shared_components/Default.aspx
//
//   Download and install
//     - IVI Shared Components 2.4.2
//     - IVI.NET Shared Components 1.4.0
//
//   32-bit: IviSharedComponents_2.4.2.exe
//           IviNetSharedComponents32_Fx20_1.4.0.exe
//
//   64-bit: IviSharedComponents64_2.4.2.exe
//           IviNetSharedComponents64_Fx20_1.4.0.exe
//
//   (You always need to install BOTH packages for your architecture)
//
//--------------------------------------------------------------------------

namespace ReadSinglePower
{
    class Program
    {
        static double Watt2dBm( double dWatt )
        {
            return (dWatt < 1.0e-19) ?  -160.0 : (10.0 * (Math.Log10(dWatt) + 3.0));
        }

        static void Main(string[] args)
        {
            RsPwrMeter driver;
            try
            {
                // If an 'alias-name' has been specified (by some VISA 
                // configuration tool), we could use the alias-name here.
                // Otherwise a qualified resource string is necessary.

                // Using alias-name ("NRP18SN" is this case)
                // driver = new RsPwrMeter("NRP18SN", true, true);


                // For using USB resource strings:
                //   0x0aad  is the VendorID of Rohde & Schwarz
                //   0x0139  is the ProductID of an NRP18SN power sensor
                //   101438  is the serial number of the specific NRP18SN used for this sample
                //
                //           Other ProductIDs could be
                //           -------------------------
                //             NRP8S      ==>  0x00E2
                //             NRP8SN     ==>  0x0137
                //             NRP18S     ==>  0x0138
                //             NRP18SN    ==>  0x0139
                //             NRP33S     ==>  0x0145
                //             NRP33SN    ==>  0x0146
                //             NRP18S-10  ==>  0x0148
                //             NRP18SN-10 ==>  0x0149
                //             NRP18S-20  ==>  0x014A
                //             NRP18SN-20 ==>  0x014B
                //             NRP18S-25  ==>  0x014C
                //             NRP18SN-25 ==>  0x014D

                Console.Out.Write("Driver session initializing...");
                driver = new RsPwrMeter("USB::0x0aad::0x0139::900004::INSTR", true, true, "Simulate=False");
                Console.Out.WriteLine("Successfull");
            }
            //in case of the initializing error the driver throws an Ivi.Driver.IOException
            catch (Ivi.Driver.IOException e)
            {
                Console.Out.WriteLine("\nCannot initialize the driver session:");
                Console.Out.WriteLine(e.Message);
                Console.Out.WriteLine("\n\nPress any key to finish...");
                Console.ReadKey();
                return;
            }

            // In order to always get decimal POINTS ('.') in the formatted
            // measurement value, we use an 'en_US' culture for formatting
            CultureInfo cultEN_US = new CultureInfo("en-US", false);

            try
            {
                //setting the OPC timeout in ms (not the VISA timeout!!!)
                driver.UtilityFunctions.OPCTimeout = 600;

                //test the sensor
                Console.Out.Write("\nSensor testing...");
                string testResult = driver.Channel["CH1"].Sensor.Test();
                Console.Out.WriteLine("Finished");
                Console.Out.WriteLine(testResult);

                //zeroing the sensor
                Console.Out.Write("\nSensor zeroing...");
                driver.Channel["CH1"].Zero();
                Console.Out.WriteLine("Finished");

                driver.Measurement.Channel["CH1"].Continuous = false;
                driver.Channel["CH1"].Frequency = 1e9;
                driver.Channel["CH1"].Mode = MeasurementMode.ContAv;
                driver.Channel["CH1"].Averaging.Enabled = false;
                driver.Channel["CH1"].Trigger.Source = TriggerSource.Bus;
                driver.Measurement.Channel["CH1"].Initiate();
                driver.Measurement.SendSoftwareTrigger();

                // Fetch() returns an array of data. When doing measurements
                // with a single result, only the first array-element contains
                // valid data
                double[] result = driver.Measurement.Fetch();
                Console.Out.WriteLine("Measured power: {0} dBm", Watt2dBm(result[0]).ToString("0.000", cultEN_US));
            }
            
            //driver IO error
            catch (Ivi.Driver.IOException e)
            {
                Console.Out.WriteLine("Driver error occured:");
                Console.Out.WriteLine(e.Message);
            }

            //driver function is not supported by this sensor
            catch (Ivi.Driver.OperationNotSupportedException e)
            {
                Console.Out.WriteLine("Instrument doesn't support the function:");
                Console.Out.WriteLine(e.Message);
            }

            //when a task takes longer than the defined OPC timeout
            catch (Ivi.Driver.MaxTimeExceededException e)
            {
                Console.Out.WriteLine("Operation took longer than the defined OPC timeout ({0} ms):", driver.UtilityFunctions.OPCTimeout);
                Console.Out.WriteLine(e.Message);
            }

            //if the instrument returns an error in the error queue
            catch (Ivi.Driver.InstrumentStatusException e)
            {
                Console.Out.WriteLine("Instrument system error occured:");
                Console.Out.WriteLine(e.Message);
                
            }

            finally
            {
                Console.Out.WriteLine("\n\nPress any key to finish...");
                Console.ReadKey();
            }
        }
    }
}
